@@ -15,6 +15,7 @@ import pyodbc
 
 __all__ = ['Virtuoso', 'resolve']
 
+from virtuoso.common import READ_COMMITTED
 log = __import__("logging").getLogger(__name__)
 
 ## hack to change BNode's random identifier generator to be
@@ -36,7 +37,8 @@ BNode.__new__ = __bnode_new__
 ## end hack
 
 import re
-_construct_re = re.compile('^[ \t\r\n]*(CONSTRUCT|DESCRIBE)', re.IGNORECASE)
+_construct_re = re.compile('^([ \t\r\n]*PREFIX [^ \t]*: <[^>]*>)*[ \t\r\n]*(CONSTRUCT|DESCRIBE)', re.IGNORECASE)
+
 
 class Virtuoso(Store):
     context_aware = True
@@ -50,13 +52,17 @@ class Virtuoso(Store):
         self.__dsn = dsn
         try:
             self._connection = pyodbc.connect(dsn)
-            self._cursor = self._connection.cursor()
             log.info("Virtuoso Store Connected: %s" % dsn)
             return VALID_STORE
         except:
             log.error("Virtuoso Connection Failed:\n%s" % format_exc())
             return NO_STORE
 
+    def cursor(self, isolation=READ_COMMITTED):
+        cursor = self._connection.cursor()
+        cursor.execute("SET TRANSACTION ISOLATION LEVEL %s" % isolation)
+        return cursor
+
     def close(self, commit_pending_transaction=False):
         if commit_pending_transaction:
             self.commit()
@@ -67,18 +73,19 @@ class Virtuoso(Store):
     def clone(self):
         return Virtuoso(self.__dsn)
 
-    def query(self, q):
+    def query(self, q, cursor=None, *av, **kw):
         log.debug(q)
+        if not cursor:
+            cursor = self.cursor(*av, **kw)
         try:
-            return self._cursor.execute(q.encode('utf-8')) #str(q))
+            return cursor.execute(q.encode('utf-8')) #str(q))
         except:
             log.error(u"Error Executing: %s" % q)
             raise
 
-    def sparql_query(self, q):
+    def sparql_query(self, q, *av, **kw):
         def _construct(results):
             # virtuoso handles construct by returning turtle
-            resolver = self._connection.cursor()
             for result, in results:
                 g = Graph()
                 turtle = result[0]
@@ -86,12 +93,12 @@ class Virtuoso(Store):
             return g
 
         def _cursor(results):
-            resolver = self._connection.cursor()
+            resolver = self.cursor()
             for result in results:
                 yield [resolve(resolver, x) for x in result]
             resolver.close()
 
-        results = self.query(u'SPARQL define output:valmode "LONG" ' + q)
+        results = self.query(u'SPARQL define output:valmode "LONG" ' + q, *av, **kw)
         if _construct_re.match(q):
             return _construct(results)
         else:
@@ -217,6 +224,8 @@ def resolve(resolver, args):
                        datatype=URIRef("http://www.w3.org/2001/XMLSchema#dateTime"))
     if dvtype == pyodbc.VIRTUOSO_DV_DATE:
         return Literal(value, datatype=URIRef("http://www.w3.org/2001/XMLSchema#date"))
+    if dvtype == 204: ## XXX where is this const!?
+        return None
     log.warn("Unhandled SPASQL DV type: %d for %s" % (dvtype, value))
     return Literal(value)
 
